function [MTime,MSim,MPotential,MActive,MRestingOx,MReduced,MPeroxy,MXstate,MConstants] = BMCO_CA_P_deadend_fully_reduced(MConstants)

% BMCO_CA_bridge
%   Simulations of chronoamperometry traces for the catalytic cycle of blue
%   multicopper oxidases assuming assuming a deadend from Peroxy state
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%
%   Output
%       MTime
%           Time vector in seconds
%       MSim
%           Current density vector in A cm^-2
%       MPotential
%           Potential vector in V vs SHE
%       MActive
%           Fraction of enzyme in Active (native intermediate) form (vector)
%       MRestingOx
%           Fraction of enzyme in Resting Oxidized (resting oxidized) form (vector)
%       MReduced
%           Fraction of enzyme in reduced form (vector)
%       MPeroxy
%           Fraction of enzyme in peroxy form (vector)
%       MXstate
%           Fraction of enzyme in "X-state" form (vector)
%       MConstants
%           Export copy of constants used in simulation (structure)
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%       .alpha
%           Electron transfer coefficient
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CA_Emeasure
%           Set potential for CA in V vs SHE
%       .CA_PretreatTime
%           Time potential held at Emeasure before open circuit
%           in seconds
%       .CA_HoldTime
%           Time at open circuit in seconds
%       .CA_ReactTime
%           Time after open circuit at Emeasure in seconds
%       .CA_ExpTimeStep
%           Time spacing on experimental data set in seconds (for
%           matching to simulation)
%
%       .pKa_a
%           pH of average rate for reaction alpha (Native to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Native to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)


% _______Check for variables in MConstants structure and set defaults if not present________

try
    isstruct(MConstants);
catch ME %#ok<NASGU>
    MConstants=struct;
end

if not(isfield(MConstants,'T')), MConstants.T=298;end;
if not(isfield(MConstants,'pH')), MConstants.pH=4;end;
if not(isfield(MConstants,'alpha')), MConstants.alpha=0.5;end;

if not(isfield(MConstants,'EnzConc')), MConstants.EnzConc=1e-12;end;

if not(isfield(MConstants,'k_alphaMax')), MConstants.k_alphaMax=2.5e3;end;
if not(isfield(MConstants,'k_betaforward')), MConstants.k_betaforward=1e-3;end;
if not(isfield(MConstants,'k_betaback')), MConstants.k_betaback=7.1e-4;end;
if not(isfield(MConstants,'k_gammaMax')), MConstants.k_gammaMax=3e-2;end;
if not(isfield(MConstants,'k_deltaMax')), MConstants.k_deltaMax=6e3;end;
if not(isfield(MConstants,'k_epsilonMax')), MConstants.k_epsilonMax=3.5e2;end;
if not(isfield(MConstants,'k_xforwardMax')), MConstants.k_xforwardMax=7e-3;end;
if not(isfield(MConstants,'k_xbackMax')), MConstants.k_xbackMax=1e-1;end;

if not(isfield(MConstants,'E_alpha')), MConstants.E_alpha=0.7;end;
if not(isfield(MConstants,'E_gamma')), MConstants.E_gamma=0.7;end;

if not(isfield(MConstants,'CA_Emeasure')), MConstants.CA_Emeasure=0.4;end;
if not(isfield(MConstants,'CA_PretreatTime')), MConstants.CA_PretreatTime=500;end;
if not(isfield(MConstants,'CA_HoldTime')), MConstants.CA_HoldTime=500;end;
if not(isfield(MConstants,'CA_ReactTime')), MConstants.CA_ReactTime=500;end;
if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

if not(isfield(MConstants,'CV_Estart')), MConstants.CV_Estart=0.9;end;
if not(isfield(MConstants,'CV_Eend')), MConstants.CV_Eend=0.4;end;
if not(isfield(MConstants,'CV_ScanRate')), MConstants.CV_ScanRate=0.01;end;
if not(isfield(MConstants,'CV_NScans')), MConstants.CV_NScans=3;end;

if not(isfield(MConstants,'pKa_alpha')), MConstants.pKa_alpha=4.6;end;
if not(isfield(MConstants,'pKa_beta')), MConstants.pKa_beta=4.6;end;
if not(isfield(MConstants,'pKa_gamma')), MConstants.pKa_gamma=20;end;
if not(isfield(MConstants,'pKa_delta')), MConstants.pKa_delta=20;end;
if not(isfield(MConstants,'pKa_epsilon')), MConstants.pKa_epsilon=20;end;
if not(isfield(MConstants,'pKa_x')), MConstants.pKa_x=5.1;end;

if not(isfield(MConstants,'InactiveFxn')), MConstants.InactiveFxn=[0.55,-0.02,0.02];end;

if not(isfield(MConstants,'TimeStep')), MConstants.TimeStep=1e-3;end;
if not(isfield(MConstants,'SteadyState')), MConstants.SteadyState=0;end;
if not(isfield(MConstants,'ShowFigure')), MConstants.ShowFigure=0;end;
if not(isfield(MConstants,'SaveFigure')), MConstants.SaveFigure=0;end;


% ______Time matrices for plotting (automatically calculated)______

ExperimentTime = MConstants.CA_PretreatTime+MConstants.CA_HoldTime+MConstants.CA_ReactTime; % Seconds
MTime = [0:MConstants.CA_ExpTimeStep:ExperimentTime]'; %#ok<NBRAK>

GraphRes = length(MTime); % Number of points on graph


% ______Calculated parameters for FTD simulation______

MConstants.f = MConstants.alpha*96485.34/MConstants.T/8.314472; % in V^-1

malphaMax = MConstants.TimeStep*MConstants.k_alphaMax*(1-1/(1+10^(MConstants.pKa_alpha-MConstants.pH))); % Active Oxidized to Reduced
mbetaforwardMax = MConstants.TimeStep*MConstants.k_betaforward*(1-1/(1+10^(MConstants.pKa_beta-MConstants.pH))); % Active Oxidized to Resting Oxidized
mbetabackMax = MConstants.TimeStep*MConstants.k_betaback; % Resting Oxidized to Active Oxidized
mgammaMax = MConstants.TimeStep*MConstants.k_gammaMax*(1-1/(1+10^(MConstants.pKa_gamma-MConstants.pH))); % Resting Oxidized to Reduced
mdeltaMax = MConstants.TimeStep*MConstants.k_deltaMax*(1-1/(1+10^(MConstants.pKa_delta-MConstants.pH))); % Reduced to Peroxy
mepsilonMax = MConstants.TimeStep*MConstants.k_epsilonMax*(1-1/(1+10^(MConstants.pKa_epsilon-MConstants.pH))); % Peroxy to Active Oxidized
mxforwardMax = MConstants.TimeStep*MConstants.k_xforwardMax; % Peroxy to Xstate
mxbackMax =MConstants.TimeStep*MConstants.k_xbackMax*(1-1/(1+10^(MConstants.pKa_x-MConstants.pH))); % Xstate to Peroxy

EO = 1.223-0.12-(log(10)*(MConstants.pH)/MConstants.f); % Approximate value of OCP at different pH; 0.1V overpotential included...


% ______Set zero matricies to speed up processing______

SaveSteps = GraphRes;
WorkSteps = ExperimentTime/MConstants.TimeStep/(GraphRes-1);
MPotential = zeros(GraphRes,1); % Potential matrix

MActive = zeros(GraphRes,1); % xA matrix
MRestingOx = zeros(GraphRes,1); % xI matrix
MReduced = zeros(GraphRes,1); % xR matrix
MPeroxy = zeros(GraphRes,1); % xP matrix
MXstate = zeros(GraphRes,1); % xX matrix

MSim = zeros(GraphRes,1); % Simulated current matrix


% ______Calculate starting values______

Time = min(MTime); %#ok<NASGU>
Potential = MConstants.CA_Emeasure;

MActive(1)=0;
MReduced(1)=1;
MPeroxy(1)=0;
MRestingOx(1)=0;
MXstate(1)=0;
MSim(1)=0;
xA = MActive(1);
xR = MReduced(1);
xP = MPeroxy(1);
xI = MRestingOx(1);
xX = MXstate(1);

% ______Main loop______

for m=1:(SaveSteps-1)
    for l=1:WorkSteps
        % Advance time
        Time=((m-1)*WorkSteps+l+1)*MConstants.TimeStep;
       
        % Set dimensionless rate constants
        malpha=malphaMax/(1.0+exp(MConstants.f*(Potential-MConstants.E_alpha)));
        mbetaforward=mbetaforwardMax;
        mbetaback=mbetabackMax;
        mgamma=mgammaMax/(1.0+exp(MConstants.f*(Potential-MConstants.E_gamma)));
        mdelta=mdeltaMax;
        mepsilon=mepsilonMax;
        mxforward=mxforwardMax;
        mxback=mxbackMax;
              
        % Set the potential
        if Time > MConstants.CA_PretreatTime && Time < (MConstants.CA_PretreatTime+MConstants.CA_HoldTime);
           Potential = xA*EO+xI*0.79+(xX+xR)*MConstants.CA_Emeasure+xP*(EO+0.55);
        else
           Potential = MConstants.CA_Emeasure;
        end %end if statement
        
        % Set the implied rate constants
        if Time > MConstants.CA_PretreatTime && Time < (MConstants.CA_PretreatTime+MConstants.CA_HoldTime);
            malpha = 0;
            mgamma = 0;
        end %end if statement
         
        % Move previous step's results to "old"
        xAold = xA;
        xIold = xI;
        xRold = xR;
        xPold = xP;
        xXold = xX;

        % **BEGIN SCHEME-DEPENDENT SECTION (1/4)**
        % Calculate common denominator etc
         D = (-(1+mdelta))*(1+mbetaback+mbetaforward+mgamma+mbetaforward*mgamma+malpha*(1+mbetaback+mgamma))*mxback*mxforward+(1+mxback)*(mbetaforward*((-mdelta)*mepsilon*mgamma-mbetaback*(1+mdelta)*(1+mepsilon+mxforward))+(1+mbetaback+mgamma)*((-malpha)*mdelta*mepsilon+(1+malpha+mbetaforward)*(1+mdelta)*(1+mepsilon+mxforward)));

        xA = ((1+mdelta)*(1+mbetaback+mgamma)*(1+mepsilon+mxback+mepsilon*mxback+mxforward)*xAold+mepsilon*(1+mxback)*(xPold+mgamma*xPold+mdelta*(xPold+xRold+mgamma*(xIold+xPold+xRold)))+(1+mdelta)*mepsilon*(1+mgamma)*mxback*xXold+mbetaback*((1+mdelta)*(1+mepsilon+mxback+mepsilon*mxback+mxforward)*xIold+mepsilon*(1+mxback)*(xPold+mdelta*xPold+mdelta*xRold)+(1+mdelta)*mepsilon*mxback*xXold))/D;
        %xR = ((1+mepsilon+mxback+mepsilon*mxback+mxforward)*(mgamma*xIold+(1+mbetaback+mgamma)*xRold)+malpha*((1+mbetaback+mgamma)*(1+mepsilon+mxback+mepsilon*mxback+mxforward)*xAold+mgamma*xIold+mepsilon*mgamma*xIold+mgamma*mxback*xIold+mepsilon*mgamma*mxback*xIold+mgamma*mxforward*xIold+mepsilon*xPold+mepsilon*mgamma*xPold+mepsilon*mxback*xPold+mepsilon*mgamma*mxback*xPold+xRold+mepsilon*xRold+mgamma*xRold+mepsilon*mgamma*xRold+mxback*xRold+mepsilon*mxback*xRold+mgamma*mxback*xRold+mepsilon*mgamma*mxback*xRold+mxforward*xRold+mgamma*mxforward*xRold+mepsilon*(1+mgamma)*mxback*xXold+mbetaback*((1+mepsilon+mxback+mepsilon*mxback+mxforward)*xIold+(1+mxback+mxforward)*xRold+mepsilon*(1+mxback)*(xPold+xRold)+mepsilon*mxback*xXold))+mbetaforward*((1+mepsilon+mxback+mepsilon*mxback+mxforward)*xRold+mgamma*((1+mepsilon+mxback+mepsilon*mxback+mxforward)*xAold+xIold+xRold+(mxback+mxforward)*(xIold+xRold)+mepsilon*(1+mxback)*(xIold+xPold+xRold)+mepsilon*mxback*xXold)))/D;
        xP = ((1+mxback)*((1+mbetaback+mgamma)*xPold+mdelta*((1+mbetaback)*(xPold+xRold)+mgamma*(xIold+xPold+xRold)))+mbetaforward*(1+mxback)*(xPold+mgamma*xPold+mdelta*(xPold+xRold+mgamma*(xAold+xIold+xPold+xRold)))+mbetaforward*(1+mdelta)*(1+mgamma)*mxback*xXold+(1+mdelta)*(1+mbetaback+mgamma)*mxback*xXold+malpha*((1+mxback)*((1+mbetaback+mgamma)*xPold+mdelta*((1+mbetaback+mgamma)*xAold+xPold+xRold+(mbetaback+mgamma)*(xIold+xPold+xRold)))+(1+mdelta)*(1+mbetaback+mgamma)*mxback*xXold))/D;
        xI = (((1+mdelta+mepsilon+mdelta*mepsilon+malpha*(1+mdelta+mepsilon))*(1+mxback)+(1+malpha)*(1+mdelta)*mxforward)*xIold+mbetaforward*((1+mdelta)*(1+mepsilon+mxback+mepsilon*mxback+mxforward)*xAold+(1+mdelta)*(1+mepsilon+mxback+mepsilon*mxback+mxforward)*xIold+mepsilon*(1+mxback)*(xPold+mdelta*xPold+mdelta*xRold)+(1+mdelta)*mepsilon*mxback*xXold))/D;
        xX = (mxforward*((1+mbetaback+mgamma)*xPold+mdelta*((1+mbetaback)*(xPold+xRold)+mgamma*(xIold+xPold+xRold)))+malpha*mxforward*((1+mbetaback+mgamma)*xPold+mdelta*((1+mbetaback+mgamma)*xAold+xPold+xRold+(mbetaback+mgamma)*(xIold+xPold+xRold)))+mbetaforward*mxforward*(xPold+mgamma*xPold+mdelta*(xPold+xRold+mgamma*(xAold+xIold+xPold+xRold)))+(1+mdelta)*(1+mbetaback+mgamma)*(1+mepsilon+mxforward)*xXold+malpha*(1+mbetaback+mgamma)*(1+mdelta+mepsilon+mxforward+mdelta*mxforward)*xXold+mbetaforward*((1+mgamma)*(1+mepsilon+mxforward)+mdelta*(1+mepsilon+mgamma+mxforward+mgamma*mxforward))*xXold)/D;
        xR = 1-xA-xP-xI-xX;
        % **END SCHEME-DEPENDENT SECTION (1/4)**

        if Time <= (MTime(m+1)+(0.5*MConstants.TimeStep)) && Time > MTime(m+1)-(0.5*MConstants.TimeStep);
            iSim=((malpha/MConstants.TimeStep)*xA+(mgamma/MConstants.TimeStep)*xI)*-4.0*9.6485e4*MConstants.EnzConc*1e6; %gives value in A cm-2

            
            % Store in the plot matrices    
            MSim(m+1) = iSim;

            MPotential(m+1) = Potential;
            
            MReduced(m+1) = xR;
            MRestingOx(m+1) = xI;
            MActive(m+1) = xA;
            MPeroxy(m+1) = xP;
            MXstate(m+1) = xX;
            
        else
        end %end 'if' loop    
    end %end for 'l=1:WorkSteps' loop
    
    
    
end %end 'for m=1:(SaveSteps-1)' loop


% ______Generate figures______

if MConstants.ShowFigure
    infotext={['pK_{a,\alpha} = ', num2str(MConstants.pKa_alpha),'  pK_{a,\beta} = ',num2str(MConstants.pKa_beta),' pK_{a,\gamma} = ',num2str(MConstants.pKa_gamma),' pK_{a,\delta} = ',num2str(MConstants.pKa_delta),'  pK_{a,\epsilon} = ',num2str(MConstants.pKa_epsilon),'  pK_{a,x} = ',num2str(MConstants.pKa_x)];['k_{\alpha} = ', num2str(MConstants.k_alphaMax,'%g'),'  k_{\beta} = ',num2str(MConstants.k_betaforward,'%g'),'  k_{-\beta}= ',num2str(MConstants.k_betaback,'%g'),'  k_{\gamma} = ',num2str(MConstants.k_gammaMax,'%g')];['k_{\delta} = ',num2str(MConstants.k_deltaMax,'%g'),'  k_{\epsilon} = ',num2str(MConstants.k_epsilonMax,'%g'),'  k_{x} = ',num2str(MConstants.k_xforwardMax,'%g'),' k_{-x} = ',num2str(MConstants.k_xbackMax,'%g')];['E_{\alpha} = ', num2str(MConstants.E_alpha),'  E_{\gamma} = ',num2str(MConstants.E_gamma),'  pH = ',num2str(MConstants.pH)]};
    figure(3)
    clf;
    subplot(3,1,1)
    plot(MTime((2):length(MTime)),MSim((2):length(MSim)),'r')
    title('Simulated Current (deadend from P scheme)','FontSize',12) % **SCHEME-DEPENDENT CODE (2/4)**
    xlabel('time / s')
    ylabel('current / Acm-2')

    subplot(3,1,2)
    plot(MTime((2):length(MTime)),MPotential((2):length(MPotential)),'g')
    title('Simulated Potential (deadend from P scheme)','FontSize',12) % **SCHEME-DEPENDENT CODE (3/4)**
    xlabel('time / s')
    ylabel('potential / V vs SHE')
    
    subplot(3,1,3)
    area([MTime,MTime,MTime,MTime,MTime],[MActive,MReduced,MPeroxy,MRestingOx,MXstate]);
    ylim([0,1]);
    legend('Native','Reduced','Peroxy','Resting Ox','X State')
    title('Fractions of enzyme states (deadend from P scheme)','FontSize',12) % ** SCHEME-DEPENDENT CODE (4/4)**
    xlabel('time / s')
    ylabel('enzyme fraction')
    text(-0.1*max(MTime),-0.2,infotext,'FontSize',8)
    
    if MConstants.SaveFigure
        filename=['Fig3_',datestr(clock,'yyyy-mm-dd-HH-MM-SS')];
        saveas(gca,filename,'png')
    end
end


end