function [MPotential,MSim,MFirstDeriv,MActive,MRestingOx,MReduced,MPeroxy,MXstate,MConstants]=BMCO_CV_P_deadend(MConstants)

% BMCO_CV_complex
%   Simulations of cyclic voltammetry traces for the catalytic cycle of 
%   blue multicopper oxidases assuming a deadend from Peroxy intermediate
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%
%   Output
%       MPotential
%           Potential vector in V vs SHE
%       MSim
%           Current density vector in A cm^-2
%       MFirstDeriv
%           di/dE in A cm^-2 V^-1
%       MActive
%           Fraction of enzyme in Active (native intermediate) form (vector)
%       MRestingOx
%           Fraction of enzyme in Resting Oxidized (resting oxidized) form (vector)
%       MReduced
%           Fraction of enzyme in reduced form (vector)
%       MPeroxy
%           Fraction of enzyme in peroxy form (vector)
%       MXstate
%           Fraction of enzyme in "X-state" form (vector)
%       MConstants
%           Export copy of constants used in simulation (structure)
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%       .alpha
%           Electron transfer coefficient
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CA_Emeasure
%           Set potential for CA in V vs SHE
%       .CA_PretreatTime
%           Time potential held at Emeasure before open circuit
%           in seconds
%       .CA_HoldTime
%           Time at open circuit in seconds
%       .CA_ReactTime
%           Time after open circuit at Emeasure in seconds
%       .CA_ExpTimeStep
%           Time spacing on experimental data set in seconds (for
%           matching to simulation)
%
%       .CV_Estart
%           Start potential for CV in V vs SHE
%       .CV_Eend
%           Vertex potential for CV in V vs SHE
%       .CV_ScanRate
%           Rate of change in potential in V s-1
%       .CV_NScans
%           Number of potential cycles
%
%       .pKa_a
%           pH of average rate for reaction alpha (Native to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Native to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)


% _______Check for variables in MConstants structure and set defaults if not present________

try
    isstruct(MConstants);
catch ME %#ok<NASGU>
    MConstants=struct;
end

if not(isfield(MConstants,'T')), MConstants.T=298;end;
if not(isfield(MConstants,'pH')), MConstants.pH=4;end;
if not(isfield(MConstants,'alpha')), MConstants.alpha=0.5;end;

if not(isfield(MConstants,'EnzConc')), MConstants.EnzConc=1e-12;end;

if not(isfield(MConstants,'k_alphaMax')), MConstants.k_alphaMax=2.5e3;end;
if not(isfield(MConstants,'k_betaforward')), MConstants.k_betaforward=1e-3;end;
if not(isfield(MConstants,'k_betaback')), MConstants.k_betaback=7.1e-4;end;
if not(isfield(MConstants,'k_gammaMax')), MConstants.k_gammaMax=3e-2;end;
if not(isfield(MConstants,'k_deltaMax')), MConstants.k_deltaMax=6e3;end;
if not(isfield(MConstants,'k_epsilonMax')), MConstants.k_epsilonMax=3.5e2;end;
if not(isfield(MConstants,'k_xforwardMax')), MConstants.k_xforwardMax=7e-3;end;
if not(isfield(MConstants,'k_xbackMax')), MConstants.k_xbackMax=1e-1;end;

if not(isfield(MConstants,'E_alpha')), MConstants.E_alpha=0.7;end;
if not(isfield(MConstants,'E_gamma')), MConstants.E_gamma=0.7;end;

if not(isfield(MConstants,'CA_Emeasure')), MConstants.CA_Emeasure=0.4;end;
if not(isfield(MConstants,'CA_PretreatTime')), MConstants.CA_PretreatTime=500;end;
if not(isfield(MConstants,'CA_HoldTime')), MConstants.CA_HoldTime=500;end;
if not(isfield(MConstants,'CA_ReactTime')), MConstants.CA_ReactTime=500;end;
if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

if not(isfield(MConstants,'CV_Estart')), MConstants.CV_Estart=0.9;end;
if not(isfield(MConstants,'CV_Eend')), MConstants.CV_Eend=0.4;end;
if not(isfield(MConstants,'CV_ScanRate')), MConstants.CV_ScanRate=0.01;end;
if not(isfield(MConstants,'CV_NScans')), MConstants.CV_NScans=3;end;

if not(isfield(MConstants,'pKa_alpha')), MConstants.pKa_alpha=4.6;end;
if not(isfield(MConstants,'pKa_beta')), MConstants.pKa_beta=4.6;end;
if not(isfield(MConstants,'pKa_gamma')), MConstants.pKa_gamma=20;end;
if not(isfield(MConstants,'pKa_delta')), MConstants.pKa_delta=20;end;
if not(isfield(MConstants,'pKa_epsilon')), MConstants.pKa_epsilon=20;end;
if not(isfield(MConstants,'pKa_x')), MConstants.pKa_x=5.1;end;

if not(isfield(MConstants,'InactiveFxn')), MConstants.InactiveFxn=[0.55,-0.02,0.02];end;

if not(isfield(MConstants,'TimeStep')), MConstants.TimeStep=1e-3;end;
if not(isfield(MConstants,'SteadyState')), MConstants.SteadyState=0;end;
if not(isfield(MConstants,'ShowFigure')), MConstants.ShowFigure=0;end;
if not(isfield(MConstants,'SaveFigure')), MConstants.SaveFigure=0;end;


% ______Computer-determined parameters______

MConstants.f = MConstants.alpha*96485.34/MConstants.T/8.314472; % in V^-1

ScanDirection=sign(MConstants.CV_Eend-MConstants.CV_Estart);
CycleTime=2*abs(MConstants.CV_Eend-MConstants.CV_Estart)/MConstants.CV_ScanRate; % Time for 1 CV in seconds
tau = MConstants.f*MConstants.CV_ScanRate*MConstants.TimeStep;

malphaMax = (MConstants.k_alphaMax*(1-1/(1+10^(MConstants.pKa_alpha-MConstants.pH))))/MConstants.f/MConstants.CV_ScanRate; % Active Oxidized to Reduced
mbetaforwardMax = (MConstants.k_betaforward*(1-1/(1+10^(MConstants.pKa_beta-MConstants.pH))))/MConstants.f/MConstants.CV_ScanRate; % Active Oxidized to Resting Oxidized
mbetabackMax = MConstants.k_betaback/MConstants.f/MConstants.CV_ScanRate; % Resting Oxidized to Active Oxidized
mgammaMax = (MConstants.k_gammaMax*(1-1/(1+10^(MConstants.pKa_gamma-MConstants.pH))))/MConstants.f/MConstants.CV_ScanRate; % Resting Oxidized to Reduced
mdeltaMax = MConstants.k_deltaMax*(1-1/(1+10^(MConstants.pKa_delta-MConstants.pH)))/MConstants.f/MConstants.CV_ScanRate; % Reduced to Peroxy
mepsilonMax = MConstants.k_epsilonMax*(1-1/(1+10^(MConstants.pKa_epsilon-MConstants.pH)))/MConstants.f/MConstants.CV_ScanRate; % Peroxy to Active Oxidized
mxforwardMax = MConstants.k_xforwardMax/MConstants.f/MConstants.CV_ScanRate; % Peroxy to Xstate
mxbackMax = MConstants.k_xbackMax*(1-1/(1+10^(MConstants.pKa_x-MConstants.pH)))/MConstants.f/MConstants.CV_ScanRate; % Xstate to Peroxy


% ______Set zero matricies to speed up processing______

GraphSteps = MConstants.GraphRes*MConstants.CV_NScans+1;
WorkSteps = CycleTime/MConstants.TimeStep/MConstants.GraphRes;

MTime = zeros(MConstants.GraphRes,1); % Time matrix
MPotential = zeros(MConstants.GraphRes,1); % Potential matrix
MActive = zeros(MConstants.GraphRes,1); % xA matrix
MRestingOx = zeros(MConstants.GraphRes,1); % xI matrix
MReduced = zeros(MConstants.GraphRes,1); % xR matrix
MPeroxy = zeros(MConstants.GraphRes,1); % xP matrix
MXstate = zeros(MConstants.GraphRes,1); % xX matrix

MSim = zeros(MConstants.GraphRes,1); % Simulated current matrix


% ______Calculate starting values______

Time = min(MTime);
Potential = MConstants.CV_Estart;

ScanTime = 0; %#ok<NASGU>
ScanNumber = 1;

[MActive(1),MReduced(1),MPeroxy(1),MRestingOx(1),MXstate(1),MSim(1)]=BMCO_SS_P_deadend(MConstants,Potential);  % ** SCHEME-DEPENDENT CODE (1/4)**
xA = MActive(1);
xR = MReduced(1);
xP = MPeroxy(1);
xI = MRestingOx(1);
xX = MXstate(1);

MTime(1) = Time;
MPotential(1) = Potential;


% ______Main loop______

for m=1:(GraphSteps-1)
    for l=1:WorkSteps
        % Advance time and potential
        Time=((m-1)*WorkSteps+l+1)*MConstants.TimeStep;
        ScanTime=Time-(ScanNumber-1)*CycleTime;
        if ScanTime >= CycleTime && ScanNumber < MConstants.CV_NScans  %&&=AND; ||=OR
            ScanNumber = ScanNumber + 1;
            ScanTime = ScanTime - CycleTime;
        end
        
        if (ScanTime > CycleTime/2)
            Potential = 2*MConstants.CV_Eend-MConstants.CV_Estart-ScanDirection*MConstants.CV_ScanRate*ScanTime;
        else
            Potential = MConstants.CV_Estart+ScanDirection*MConstants.CV_ScanRate*ScanTime;
        end
        
        % Set dimensionless rate constants
        malpha=malphaMax/(1.0+exp(MConstants.f*(Potential-MConstants.E_alpha)));
        mbetaforward=mbetaforwardMax;
        mbetaback=mbetabackMax;
        mgamma=mgammaMax/(1.0+exp(MConstants.f*(Potential-MConstants.E_gamma)));
        mdelta=mdeltaMax;
        mepsilon=mepsilonMax;
        mxforward=mxforwardMax;
        mxback=mxbackMax;
        
        % Move previous step's results to "old"
        xAold = xA;
        xIold = xI;
        xRold = xR;
        xPold = xP;
        xXold = xX;

        % **BEGIN SCHEME-DEPENDENT SECTION (2/4)**
        % Calculate common denominator etc
         D = (-mxback)*mxforward*tau^2*(1+mdelta*tau)*(1+tau*(malpha+mbetaback+mbetaforward+mgamma+mbetaforward*mgamma*tau+malpha*(mbetaback+mgamma)*tau))+(1+mxback*tau)*(mbetaforward*tau*((-mdelta)*mepsilon*mgamma*tau^3-mbetaback*tau*(1+mdelta*tau)*(1+(mepsilon+mxforward)*tau))+(1+(mbetaback+mgamma)*tau)*((-malpha)*mdelta*mepsilon*tau^3+(1+(malpha+mbetaforward)*tau)*(1+mdelta*tau)*(1+(mepsilon+mxforward)*tau)));

        xA = ((1+mdelta*tau)*(1+(mbetaback+mgamma)*tau)*(1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xAold+tau*(mepsilon*(1+mxback*tau)*(xPold+mgamma*tau*xPold+mdelta*tau*(xPold+xRold+mgamma*tau*(xIold+xPold+xRold)))+mepsilon*mxback*tau*(1+mdelta*tau)*(1+mgamma*tau)*xXold+mbetaback*((1+mdelta*tau)*(1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xIold+mepsilon*tau*((1+mxback*tau)*(xPold+mdelta*tau*xPold+mdelta*tau*xRold)+mxback*tau*(1+mdelta*tau)*xXold))))/D;
        %xR = ((1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*(xRold+mbetaback*tau*xRold+mgamma*tau*(xIold+xRold))+malpha*tau*((1+(mbetaback+mgamma)*tau)*(1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xAold+xRold+mbetaback*tau*((1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xIold+xRold+tau*((mxback+mxforward)*xRold+mepsilon*(1+mxback*tau)*(xPold+xRold)+mepsilon*mxback*tau*xXold))+tau*((mxback+mxforward)*xRold+mepsilon*(1+mxback*tau)*(xPold+xRold)+mepsilon*mxback*tau*xXold+mgamma*((1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xIold+xRold+tau*((mxback+mxforward)*xRold+mepsilon*(1+mxback*tau)*(xPold+xRold)+mepsilon*mxback*tau*xXold))))+mbetaforward*tau*(xRold+tau*((mepsilon+mxback+mxforward+mepsilon*mxback*tau)*xRold+mgamma*((1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xAold+xIold+xRold+tau*((mxback+mxforward)*(xIold+xRold)+mepsilon*(1+mxback*tau)*(xIold+xPold+xRold)+mepsilon*mxback*tau*xXold)))))/D;
        xP = ((1+mxback*tau)*(xPold+tau*((mbetaback+mgamma)*xPold+mdelta*(xPold+xRold+tau*(mbetaback*(xPold+xRold)+mgamma*(xIold+xPold+xRold)))))+mxback*tau*(1+mdelta*tau)*(1+(mbetaback+mgamma)*tau)*xXold+mbetaforward*tau*((1+mxback*tau)*(xPold+mgamma*tau*xPold+mdelta*tau*(xPold+xRold+mgamma*tau*(xAold+xIold+xPold+xRold)))+mxback*tau*(1+mdelta*tau)*(1+mgamma*tau)*xXold)+malpha*tau*((1+(mbetaback+mgamma)*tau)*(xPold+mxback*tau*xPold+mxback*tau*xXold)+mdelta*tau*((1+mxback*tau)*((1+(mbetaback+mgamma)*tau)*xAold+xPold+xRold+(mbetaback+mgamma)*tau*(xIold+xPold+xRold))+mxback*tau*(1+(mbetaback+mgamma)*tau)*xXold)))/D;
        xI = (((1+mdelta*tau)*(1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))+malpha*tau*(1+tau*(mdelta+mepsilon+mxback+mxforward+mepsilon*mxback*tau+mdelta*(mxback+mxforward)*tau)))*xIold+mbetaforward*tau*((1+mdelta*tau)*(1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xAold+(1+mdelta*tau)*(1+tau*(mepsilon+mxback+mxforward+mepsilon*mxback*tau))*xIold+mepsilon*tau*((1+mxback*tau)*(xPold+mdelta*tau*xPold+mdelta*tau*xRold)+mxback*tau*(1+mdelta*tau)*xXold)))/D;
        xX = (mxforward*tau*(xPold+tau*((mbetaback+mgamma)*xPold+mdelta*(xPold+xRold+tau*(mbetaback*(xPold+xRold)+mgamma*(xIold+xPold+xRold)))))+(1+mdelta*tau)*(1+(mbetaback+mgamma)*tau)*(1+(mepsilon+mxforward)*tau)*xXold+malpha*tau*(mxforward*tau*(xPold+tau*((mbetaback+mgamma)*xPold+mdelta*((1+(mbetaback+mgamma)*tau)*xAold+xPold+xRold+(mbetaback+mgamma)*tau*(xIold+xPold+xRold))))+(1+(mbetaback+mgamma)*tau)*(1+tau*(mdelta+mepsilon+mxforward+mdelta*mxforward*tau))*xXold)+mbetaforward*tau*((1+mgamma*tau)*(mxforward*tau*xPold+xXold+(mepsilon+mxforward)*tau*xXold)+mdelta*tau*(mxforward*tau*(xPold+xRold+mgamma*tau*(xAold+xIold+xPold+xRold))+xXold+tau*(mepsilon+mgamma+mxforward+mgamma*mxforward*tau)*xXold)))/D;
        xR = 1-xA-xP-xI-xX;
        % **END SCHEME-DEPENDENT SECTION (2/4)**
        
    end
    
    % Store in the plot matrix

    iSim=((malpha*xA+mgamma*xI)*MConstants.CV_ScanRate*MConstants.f)*-4.0*9.6485e4*MConstants.EnzConc*1e6; %gives value in A cm-2

    MTime(m+1) = Time;
    MPotential(m+1) = Potential;


    MReduced(m+1) = xR;
    MRestingOx(m+1) = xI;
    MActive(m+1) = xA;
    MPeroxy(m+1) = xP;
    MXstate(m+1) = xX;
    MSim(m+1) = iSim;

end


% ______Differentiate reductive wave______

MFirstDeriv = diff(MSim)./diff(MPotential);


% ______Generate Figures______

if MConstants.ShowFigure
    infotext={['pK_{a,\alpha} = ', num2str(MConstants.pKa_alpha),'  pK_{a,\beta} = ',num2str(MConstants.pKa_beta),' pK_{a,\gamma} = ',num2str(MConstants.pKa_gamma),' pK_{a,\delta} = ',num2str(MConstants.pKa_delta),'  pK_{a,\epsilon} = ',num2str(MConstants.pKa_epsilon),'  pK_{a,x} = ',num2str(MConstants.pKa_x)];['k_{\alpha} = ', num2str(MConstants.k_alphaMax,'%g'),'  k_{\beta} = ',num2str(MConstants.k_betaforward,'%g'),'  k_{-\beta}= ',num2str(MConstants.k_betaback,'%g'),'  k_{\gamma} = ',num2str(MConstants.k_gammaMax,'%g')];['k_{\delta} = ',num2str(MConstants.k_deltaMax,'%g'),'  k_{\epsilon} = ',num2str(MConstants.k_epsilonMax,'%g'),'  k_{x} = ',num2str(MConstants.k_xforwardMax,'%g'),' k_{-x} = ',num2str(MConstants.k_xbackMax,'%g')];['E_{\alpha} = ', num2str(MConstants.E_alpha),'  E_{\gamma} = ',num2str(MConstants.E_gamma),'  pH = ',num2str(MConstants.pH)]};
    figure(4)
    clf;
    subplot(2,1,1), plot(MPotential((2):length(MPotential)),MSim((2):length(MSim)),'r')
    title('Simulated Current (deadend from P scheme)','FontSize',12)  % ** SCHEME-DEPENDENT CODE (3/4)**
    xlabel('potential / V vs SHE')
    ylabel('current / A cm^{-2}')

    subplot(2,1,2), plot(MPotential((3):length(MPotential)),MFirstDeriv((2):length(MFirstDeriv)),'r')
    title('First derivative (deadend from P scheme)','FontSize',12)  % ** SCHEME-DEPENDENT CODE (4/4)**
    xlabel('potential / V vs SHE')
    ylabel('di/dE / A cm^{-2} V^{-1}')
    minx=min(MPotential((3):length(MPotential)));maxx=max(MPotential((3):length(MPotential)));
    miny=min(MFirstDeriv((2):length(MFirstDeriv)));maxy=max(MFirstDeriv((2):length(MFirstDeriv)));
    text(minx+0.05*(maxx-minx),miny+0.85*(maxy-miny),infotext,'FontSize',8)
    
    if MConstants.SaveFigure
        filename=['Fig4_',datestr(clock,'yyyy-mm-dd-HH-MM-SS')];
        saveas(gca,filename,'fig')
        saveas(gca,filename,'png')
    end


end


end

