function [E,pH,ReciprocalxO2,KM,VMax,RSq,ReciprocalCurrent]=BMCO_KMO2_P_deadend(MConstants,pHStart,pHEnd,pHStep,EStart,EEnd,EStep,FractionO2Low,FractionO2High,O2Steps)

% BMCO_KMO2
%   Calculates KM and VMax for a range of applied potentials and pH values 
%   from steady-state current values. Generates two-dimensional matrices 
%   of the results and three-dimensional matrices of reciprocal 
%   steady-state current. O2 concentrations are used to adjust the rate of
%   k_delta, with the value in MConstants considered to be the rate for
%   100% O2. KM and VMax calculated using Lineweaver-Burk analysis. Assumes
%   Henry's constant for O2 = 1.3e-3 M atm^-1
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%       pHStart
%           First pH value in range of pH values
%       pHEnd
%           Maximum pH value (may not be last value depending on pHstep
%           size)
%       pHStep
%           Size of increment in pH values for simulation
%       EStart
%           Starting applied electrochemical potential in Volts
%       EEnd
%           Maximum applied electrochemical potential in Volts
%       EStep
%           Number of Volts between steps for applied potential
%       FractionO2Low
%           Lowest fraction of O2 used to calculate KM (0.01 = 1%)
%       FractionO2High
%           Highest fraction of O2 used to calculate KM (1 = 100%)
%       O2Steps
%           Integer number of steps between high and low O2 values, minimum
%           two
%
%   Output
%       E
%           Index of applied potentials (one-dimensional vector)
%       pH
%           Index of pH values scanned (one-dimensional vector)
%       ReciprocalxO2
%           Index of 1/(fraction O2) (one-dimensional vector)
%       KM
%           Two-dimensional array of KM values: first index potential,
%           second index pH
%       VMax
%           Corresponding two-dimensional array of Vmax values from the 
%           Lineweaver-Burk fit: first index potential, second index pH
%       RSq
%           R-square values for the fits to find KM and Vmax (indices as
%           above)
%       ReciprocalCurrent
%           Three-dimensional matrix of 1/(steady-state current): (E index,
%           pH index, concentration index)
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CV_Estart
%           Start potential for CV in V vs SHE
%       .CV_Eend
%           Vertex potential for CV in V vs SHE
%       .CV_ScanRate
%           Rate of change in potential in V s-1
%       .CV_NScans
%           Number of potential cycles
%
%       .pKa_a
%           pH of average rate for reaction alpha (Native to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Native to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)

% _______Check for variables in MConstants structure and set defaults if not present________

try
    isstruct(MConstants);
catch ME %#ok<NASGU>
    MConstants=struct;
end

if not(isfield(MConstants,'T')), MConstants.T=298;end;
if not(isfield(MConstants,'pH')), MConstants.pH=4;end;

if not(isfield(MConstants,'EnzConc')), MConstants.EnzConc=1e-12;end;

if not(isfield(MConstants,'k_alphaMax')), MConstants.k_alphaMax=2.5e3;end;
if not(isfield(MConstants,'k_betaforward')), MConstants.k_betaforward=1e-3;end;
if not(isfield(MConstants,'k_betaback')), MConstants.k_betaback=7.1e-4;end;
if not(isfield(MConstants,'k_gammaMax')), MConstants.k_gammaMax=3e-2;end;
if not(isfield(MConstants,'k_deltaMax')), MConstants.k_deltaMax=6e3;end;
if not(isfield(MConstants,'k_epsilonMax')), MConstants.k_epsilonMax=3.5e2;end;
if not(isfield(MConstants,'k_xforwardMax')), MConstants.k_xforwardMax=7e-3;end;
if not(isfield(MConstants,'k_xbackMax')), MConstants.k_xbackMax=1e-1;end;

if not(isfield(MConstants,'E_alpha')), MConstants.E_alpha=0.7;end;
if not(isfield(MConstants,'E_gamma')), MConstants.E_gamma=0.7;end;

if not(isfield(MConstants,'CA_Emeasure')), MConstants.CA_Emeasure=0.4;end;
if not(isfield(MConstants,'CA_PretreatTime')), MConstants.CA_PretreatTime=500;end;
if not(isfield(MConstants,'CA_HoldTime')), MConstants.CA_HoldTime=500;end;
if not(isfield(MConstants,'CA_ReactTime')), MConstants.CA_ReactTime=500;end;
if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

if not(isfield(MConstants,'CV_Estart')), MConstants.CV_Estart=0.9;end;
if not(isfield(MConstants,'CV_Eend')), MConstants.CV_Eend=0.4;end;
if not(isfield(MConstants,'CV_ScanRate')), MConstants.CV_ScanRate=0.01;end;
if not(isfield(MConstants,'CV_NScans')), MConstants.CV_NScans=3;end;

if not(isfield(MConstants,'pKa_alpha')), MConstants.pKa_alpha=4.6;end;
if not(isfield(MConstants,'pKa_beta')), MConstants.pKa_beta=4.6;end;
if not(isfield(MConstants,'pKa_gamma')), MConstants.pKa_gamma=20;end;
if not(isfield(MConstants,'pKa_delta')), MConstants.pKa_delta=20;end;
if not(isfield(MConstants,'pKa_epsilon')), MConstants.pKa_epsilon=20;end;
if not(isfield(MConstants,'pKa_x')), MConstants.pKa_x=5.1;end;

if not(isfield(MConstants,'InactiveFxn')), MConstants.InactiveFxn=[0.55,4,-0.02,0.02,5.5];end;

if not(isfield(MConstants,'TimeStep')), MConstants.TimeStep=1e-3;end;
if not(isfield(MConstants,'SteadyState')), MConstants.SteadyState=1;end;
if not(isfield(MConstants,'ShowFigure')), MConstants.ShowFigure=0;end;
if not(isfield(MConstants,'SaveFigure')), MConstants.SaveFigure=0;end;

MConstants.SteadyState=1;
kHO2 = 1.2e-3; % Henry's constant for O2 in water in M per atm

% Size and zero storage matrices
StepsPerPotential = floor((EEnd-EStart+0.001)/EStep); % fudge
if ~mod((EEnd-EStart),EStep), StepsPerPotential=StepsPerPotential+1;end;
StepsPerpH = floor((pHEnd-pHStart)/pHStep);
if ~mod((pHEnd-pHStart),pHStep), StepsPerpH=StepsPerpH+1;end;
if O2Steps < 2
    O2Steps = 2;
else
    O2Steps = floor(O2Steps);
end
IO2End = 1/FractionO2Low;
IO2Start = 1/FractionO2High;
IO2Step = (IO2End-IO2Start)/(O2Steps-1);

E=zeros(StepsPerPotential,1);
pH=zeros(StepsPerpH,1);
ReciprocalxO2=zeros(O2Steps,1);
dummy=zeros(O2Steps,1);
KM=zeros(StepsPerPotential,StepsPerpH);
VMax=zeros(StepsPerPotential,StepsPerpH);
RSq=zeros(StepsPerPotential,StepsPerpH);
ReciprocalCurrent=zeros(StepsPerPotential,StepsPerpH,O2Steps); % Dimensions: (E index, pH index, concentration index)

MaxO2Rate = MConstants.k_deltaMax;

% Fill index matrices
E=(EStart:EStep:EEnd);
pH=(pHStart:pHStep:pHEnd);
ReciprocalxO2=(IO2Start:IO2Step:IO2End);

% Iterate and fill data matrices
i=1; % E index
while i<=StepsPerPotential
    j=1; % pH index
    while j<=StepsPerpH
        k=1; % IO2 index
        MConstants.pH=pH(j);
        while k<=O2Steps
            MConstants.k_deltaMax=MaxO2Rate/ReciprocalxO2(k);
            [~,~,~,~,~,Current]=BMCO_SS_P_deadend(MConstants,E(i)); % **SCHEME-DEPENDENT CODE (1/1)**
            ReciprocalCurrent(i,j,k)=1/Current;
            dummy(k)=1/Current;
            k=k+1;
        end
        z = polyfit(ReciprocalxO2,dummy',1);
        KM(i,j)=z(1)/z(2)*kHO2;
        VMax(i,j)=1/z(2);
        yfit = polyval(z,ReciprocalxO2);
        yresid = dummy' - yfit;
        SSresid = sum(yresid.^2);
        SStotal = (O2Steps-1) * var(dummy);
        RSq(i,j) = 1 - SSresid/SStotal;
        j=j+1;
    end
    i=i+1;
end


end