function [xA,xR,xP,xI,xX,SSCurrent]=BMCO_SS_P_deadend(MConstants,Potential)

% BMCO_SS
%   Calculates steady-state enzyme fractions and current for a given set of
%   rate constants and applied potential. If MConstants.SteadyState is 0,
%   then open-circuit is assumed (no electrocatalysis).
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%       Potential
%           Applied potential for measurements undergoing electrocatalysis
%
%   Output
%       xA
%           Fraction of enzyme in Active (native intermediate) form
%       xR
%           Fraction of enzyme in reduced form
%       xP
%           Fraction of enzyme in peroxy form
%       xI
%           Fraction of enzyme in Resting Oxidized (resting oxidized) form
%       xX
%           Fraction of enzyme in "X-state" form
%       SSCurrent
%           Steady-state current (0 if MConstants.SteadyState=0)
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CV_Estart
%           Start potential for CV in V vs SHE
%       .CV_Eend
%           Vertex potential for CV in V vs SHE
%       .CV_ScanRate
%           Rate of change in potential in V s-1
%       .CV_NScans
%           Number of potential cycles
%
%       .pKa_a
%           pH of average rate for reaction alpha (Native to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Native to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)

% NB: no checks on presence of variables in MConstants structure is here to
% avoid slowing code when this function is repeatedly called

% The timestep is gone because it cancels out and it speeds up processing
MConstants.f = MConstants.alpha*96485.34/MConstants.T/8.314472; % in V^-1
malphaMax = MConstants.k_alphaMax*(1-1/(1+10^(MConstants.pKa_alpha-MConstants.pH))); % Active Oxidized to Reduced
mbetaforwardMax = MConstants.k_betaforward*(1-1/(1+10^(MConstants.pKa_beta-MConstants.pH))); % Active Oxidized to Resting Oxidized
mbetabackMax = MConstants.k_betaback; % Resting Oxidized to Active Oxidized
mgammaMax = MConstants.k_gammaMax*(1-1/(1+10^(MConstants.pKa_alpha-MConstants.pH))); % Resting Oxidized to Reduced
mdeltaMax = MConstants.k_deltaMax*(1-1/(1+10^(MConstants.pKa_delta-MConstants.pH))); % Reduced to Peroxy
mepsilonMax = MConstants.k_epsilonMax*(1-1/(1+10^(MConstants.pKa_epsilon-MConstants.pH))); % Peroxy to Active Oxidized
mxforwardMax = MConstants.k_xforwardMax; % Peroxy to Xstate
mxbackMax = MConstants.k_xbackMax*(1-1/(1+10^(MConstants.pKa_x-MConstants.pH))); % Xstate to Peroxy

% **BEGIN SCHEME-DEPENDENT SECTION (1/1)**
if MConstants.SteadyState
    % starting values assume steady-state catalysis prior to measurement
    malpha=malphaMax/(1.0+exp(MConstants.f*(Potential-MConstants.E_alpha)));
    mgamma=mgammaMax/(1.0+exp(MConstants.f*(Potential-MConstants.E_gamma)));

    D = ((mbetabackMax+mbetaforwardMax)*mdeltaMax*mepsilonMax+(mdeltaMax*mepsilonMax+mbetaforwardMax*(mdeltaMax+mepsilonMax))*mgamma)*mxbackMax+mbetaforwardMax*mdeltaMax*mgamma*mxforwardMax+malpha*(mbetabackMax+mgamma)*(mepsilonMax*mxbackMax+mdeltaMax*(mxbackMax+mxforwardMax));

    xA = (mdeltaMax*mepsilonMax*(mbetabackMax+mgamma)*mxbackMax)/D;
    xR = (mepsilonMax*(mbetaforwardMax*mgamma+malpha*(mbetabackMax+mgamma))*mxbackMax)/D;
    xP = (mdeltaMax*(mbetaforwardMax*mgamma+malpha*(mbetabackMax+mgamma))*mxbackMax)/D;
    xI = (mbetaforwardMax*mdeltaMax*mepsilonMax*mxbackMax)/D;
    xX = 1-xA-xR-xP-xI;
    SSCurrent = (malpha*xA+mgamma*xI)*-4.0*9.6485e4*MConstants.EnzConc*1e6; %gives value in A cm-2
else
    % these starting values assume OC ie no electron flow
    D = mbetabackMax + mbetaforwardMax;
    
    xA = mbetabackMax/D;
    xR = 0;
    xP = 0;
    xI = 1-xA;
    xX = 0;
    SSCurrent = 0;
end
% **END SCHEME-DEPENDENT SECTION (1/1)**

end