function [pH,MaxCurrent,SSCurrent,XFraction,MaxEFirstDeriv,FractionActivated]=BMCO_pH_profile_P_deadend(MConstants, pHstart, pHend, pHstep)

% BMCO_pH_profile
%   Simulates pH dependence of current (catalytic activity profile, cf.
%   Fig. 5A), Ecat (Fig. 5B), apparent X-state (Fig. 4B), and relative
%   amount of enzyme activation when scanning CVs (cf. Figs 1 & 8).
%   Generates plots of same if MConstants.ShowFigure=1
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%       pHstart
%           First pH value in range of pH values
%       pHend
%           Maximum pH value (may not be last value depending on pHstep
%           size)
%       pHstep
%           Size of increment in pH values for simulation
%
%   Output
%       pH
%           Index of pH values scanned (one-dimensional vector)
%       MaxCurrent
%           Largest current measured in trace (can be larger than
%           steady-state current if a significant X-state component exists
%           at steady-state) (one-dimensional vector, one entry per pH
%           value)
%       XFraction
%           Apparent fraction of X-state, measured as one minus the ratio
%           of the current 20 seconds after potential step and the
%           steady-state current (one-dimensional vector, one entry per pH
%           value)
%       MaxEFirstDeriv
%           Maximum point in first derivative (di/dE) of cyclic voltammetry
%           traces (a.k.a. E_cat) (one-dimensional vector, one entry per pH
%           value)
%       FractionActivated
%           Ratio of current at low-potential vertex in cyclic voltammogram
%           to the steady-state current (two-dimension matrix: (pH
%           value,scan number))
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%       .alpha
%           Electron transfer coefficient
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CV_Estart
%           Start potential for CV in V vs SHE
%       .CV_Eend
%           Vertex potential for CV in V vs SHE
%       .CV_ScanRate
%           Rate of change in potential in V s-1
%       .CV_NScans
%           Number of potential cycles
%
%       .pKa_a
%           pH of average rate for reaction alpha (Native to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Native to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)


% _______Check for variables in MConstants structure and set defaults if not present________

try
    isstruct(MConstants);
catch ME %#ok<NASGU>
    MConstants=struct;
end

if not(isfield(MConstants,'T')), MConstants.T=298;end;
if not(isfield(MConstants,'pH')), MConstants.pH=4;end;
if not(isfield(MConstants,'alpha')), MConstants.alpha=0.5;end;

if not(isfield(MConstants,'EnzConc')), MConstants.EnzConc=1e-12;end;

if not(isfield(MConstants,'k_alphaMax')), MConstants.k_alphaMax=2.5e3;end;
if not(isfield(MConstants,'k_betaforward')), MConstants.k_betaforward=1e-3;end;
if not(isfield(MConstants,'k_betaback')), MConstants.k_betaback=7.1e-4;end;
if not(isfield(MConstants,'k_gammaMax')), MConstants.k_gammaMax=3e-2;end;
if not(isfield(MConstants,'k_deltaMax')), MConstants.k_deltaMax=6e3;end;
if not(isfield(MConstants,'k_epsilonMax')), MConstants.k_epsilonMax=3.5e2;end;
if not(isfield(MConstants,'k_xforwardMax')), MConstants.k_xforwardMax=7e-3;end;
if not(isfield(MConstants,'k_xbackMax')), MConstants.k_xbackMax=1e-1;end;

if not(isfield(MConstants,'E_alpha')), MConstants.E_alpha=0.7;end;
if not(isfield(MConstants,'E_gamma')), MConstants.E_gamma=0.7;end;

if not(isfield(MConstants,'CA_Emeasure')), MConstants.CA_Emeasure=0.4;end;
if not(isfield(MConstants,'CA_PretreatTime')), MConstants.CA_PretreatTime=500;end;
if not(isfield(MConstants,'CA_HoldTime')), MConstants.CA_HoldTime=500;end;
if not(isfield(MConstants,'CA_ReactTime')), MConstants.CA_ReactTime=500;end;
if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

if not(isfield(MConstants,'CV_Estart')), MConstants.CV_Estart=0.9;end;
if not(isfield(MConstants,'CV_Eend')), MConstants.CV_Eend=0.4;end;
if not(isfield(MConstants,'CV_ScanRate')), MConstants.CV_ScanRate=0.01;end;
if not(isfield(MConstants,'CV_NScans')), MConstants.CV_NScans=3;end;

if not(isfield(MConstants,'pKa_alpha')), MConstants.pKa_alpha=4.6;end;
if not(isfield(MConstants,'pKa_beta')), MConstants.pKa_beta=4.6;end;
if not(isfield(MConstants,'pKa_gamma')), MConstants.pKa_gamma=20;end;
if not(isfield(MConstants,'pKa_delta')), MConstants.pKa_delta=20;end;
if not(isfield(MConstants,'pKa_epsilon')), MConstants.pKa_epsilon=20;end;
if not(isfield(MConstants,'pKa_x')), MConstants.pKa_x=5.1;end;

if not(isfield(MConstants,'InactiveFxn')), MConstants.InactiveFxn=[0.55,4,-0.02,0.02,5.5];end;

if not(isfield(MConstants,'TimeStep')), MConstants.TimeStep=1e-3;end;
if not(isfield(MConstants,'SteadyState')), MConstants.SteadyState=0;end;
if not(isfield(MConstants,'ShowFigure')), MConstants.ShowFigure=0;end;
if not(isfield(MConstants,'SaveFigure')), MConstants.SaveFigure=0;end;


% Zero storage matrices
TotalSteps = floor((pHend-pHstart)/pHstep);
if ~mod((pHend-pHstart),pHstep), TotalSteps=TotalSteps+1;end;
pH=zeros(TotalSteps,1);
MaxCurrent=zeros(TotalSteps,1);
XFraction=zeros(TotalSteps,1);
SSCurrent=zeros(TotalSteps,1);

FractionActivated=zeros(TotalSteps,MConstants.CV_NScans);
MaxEFirstDeriv=zeros(TotalSteps,1);

MConstants.f = MConstants.alpha*96485.34/MConstants.T/8.314472; % in V^-1
MConstants.pH=pHstart;
i=1;
while (MConstants.pH<=max(pHstart,pHend) && MConstants.pH>=min(pHstart,pHend))
    pH(i)=MConstants.pH;
    
    % Chronoamperometry data points
    [~,Current,~,~,~,~,~,~,Const]=BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (1/6)**
    [~,~,~,~,~,SSCurrent(i)]=BMCO_SS_P_deadend(MConstants,MConstants.CA_Emeasure); % **SCHEME-DEPENDENT CODE (2/6)**
    SSCurrent(i)=-SSCurrent(i);
    MaxCurrent(i)=-min(Current((2):length(Current)));
    XFraction(i)=1+Current(floor((Const.CA_PretreatTime+Const.CA_HoldTime+20)/...
        (Const.CA_PretreatTime+Const.CA_HoldTime+Const.CA_ReactTime)*...
        (length(Current)-1))+1)/SSCurrent(i);
    
    % Cyclic voltammetry data points
    [Potential,Current,Deriv,~,~,~,~,~,Const]=BMCO_CV_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (3/6)**
    for j=1:MConstants.CV_NScans
       FractionActivated(i,j)=-Current(floor((length(Current)-1)*(j-.5)/MConstants.CV_NScans))/SSCurrent(i); % Only works when starting from high potential and sweeping to low potential
    end
    MaxEFirstDeriv(i)=Potential(find(Deriv==max(Deriv),1)); % Max in first derivative from first scan
    
    % Increment
    MConstants.pH=MConstants.pH+pHstep;
    i=i+1;
end

% Generate CA figures

infotext={['pK_{a,\alpha} = ', num2str(MConstants.pKa_alpha),'  pK_{a,\beta} = ',num2str(MConstants.pKa_beta),' pK_{a,\gamma} = ',num2str(MConstants.pKa_gamma),' pK_{a,\delta} = ',num2str(MConstants.pKa_delta),'  pK_{a,\epsilon} = ',num2str(MConstants.pKa_epsilon),'  pK_{a,x} = ',num2str(MConstants.pKa_x)];['k_{\alpha} = ', num2str(MConstants.k_alphaMax,'%g'),'  k_{\beta} = ',num2str(MConstants.k_betaforward,'%g'),'  k_{-\beta}= ',num2str(MConstants.k_betaback,'%g'),'  k_{\gamma} = ',num2str(MConstants.k_gammaMax,'%g')];['k_{\delta} = ',num2str(MConstants.k_deltaMax,'%g'),'  k_{\epsilon} = ',num2str(MConstants.k_epsilonMax,'%g'),'  k_{x} = ',num2str(MConstants.k_xforwardMax,'%g'),' k_{-x} = ',num2str(MConstants.k_xbackMax,'%g')];['E_{\alpha} = ', num2str(MConstants.E_alpha),'  E_{\gamma} = ',num2str(MConstants.E_gamma)]};
figure(1);
clf;
subplot(2,1,1)
hold on;
plot(pH,MaxCurrent,'-or')
plot(pH,SSCurrent,'-xb')
legend('Highest current magnitude','Final current magnitude')
title('pH dependence on catalytic activity (deadend from P scheme)','FontSize',12) % **SCHEME-DEPENDENT CODE (4/6)**
xlabel('pH')
ylabel('|current density| / A cm^{-2}')
subplot(2,1,2)
plot(pH,XFraction,'-ob');
xlabel('pH');
ylabel('1-i_{1520}/i_{lim}');
title('Electrochemical signature of X state after hold','FontSize',12)
minx=min(pH);maxx=max(pH);
miny=min(XFraction);maxy=max(XFraction);
text(minx+0.9*(maxx-minx),miny+0.90*(maxy-miny),infotext,'FontSize',8,'HorizontalAlignment','right')
if MConstants.SaveFigure
    filename=['Fig1_',datestr(clock,'yyyy-mm-dd-HH-MM-SS')];
    saveas(gca,filename,'fig')
    saveas(gca,filename,'png')
end

% Generate CV figures

figure(2);
clf;
subplot(2,1,1)
hold on;
plot(pH,MaxEFirstDeriv,'-v')
xlabel('pH')
ylabel('potential / V vs SHE')
legend('potential at maximum in di/dE')
title('Potential of the maximum in the first derivative of the CV (deadend from P scheme)','FontSize',12) % **SCHEME-DEPENDENT CODE (5/6)**

subplot(2,1,2)
plot(pH,FractionActivated,'-^');
xlabel('pH')
ylabel('Fraction activated')
title('Fraction of maximum current observed at low-potential vertex (deadend from P scheme)','FontSize',12) % **SCHEME-DEPENDENT CODE (6/6)**
miny=min(FractionActivated(1));maxy=max(FractionActivated(MConstants.CV_NScans));
text(minx+0.9*(maxx-minx),miny+0.1*(maxy-miny),infotext,'FontSize',8,'HorizontalAlignment','right')
PlotLegend=cell(Const.CV_NScans,1);
for i=1:Const.CV_NScans
    PlotLegend(i)=cellstr(sprintf('Scan %d',i));
end
legend(PlotLegend,'Location','Best');

%end
legend('show')
if MConstants.SaveFigure
    filename=['Fig2_',datestr(clock,'yyyy-mm-dd-HH-MM-SS')];
    saveas(gca,filename,'fig')
    saveas(gca,filename,'png')
end

end