function BMCO_search_P_deadend(MConstants,Order,LogStepSize)

% BMCO_search
%   Logarithmic grid search of k_alpha, k_betaforward, k_gamma and k_xback 
%   (k_betaback and k_xforward calculated based on Resting OxidizedFxn) to match
%   experimental data with a narrow tolerance on values
%
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%       Order
%           base 10 orders of magnitude on either side of specified 
%           MConstant values
%       Step
%           Number of steps, evenly spaced on log10 scale, per order of
%           magnitude
%
%   Output
%       1. Overall list of 'hits' where all criteria are matched
%       2. Folders containing related MConstants files
%       3. If MConstants.ShowFigure=1, matching figures in those folders
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%       .alpha
%           Electron transfer coefficient
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CV_Estart
%           Start potential for CV in V vs SHE
%       .CV_Eend
%           Vertex potential for CV in V vs SHE
%       .CV_ScanRate
%           Rate of change in potential in V s-1
%       .CV_NScans
%           Number of potential cycles
%
%       .pKa_a
%           pH of average rate for reaction alpha (Native to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Native to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)


% _______Check for variables in MConstants structure and set defaults if not present________

try
    isstruct(MConstants);
catch ME %#ok<NASGU>
    MConstants=struct;
end

if not(isfield(MConstants,'T')), MConstants.T=298;end;
if not(isfield(MConstants,'pH')), MConstants.pH=4;end;
if not(isfield(MConstants,'alpha')), MConstants.alpha=0.5;end;

if not(isfield(MConstants,'EnzConc')), MConstants.EnzConc=1e-12;end;

if not(isfield(MConstants,'k_alphaMax')), MConstants.k_alphaMax=2.5e3;end;
if not(isfield(MConstants,'k_betaforward')), MConstants.k_betaforward=1e-3;end;
if not(isfield(MConstants,'k_betaback')), MConstants.k_betaback=7.1e-4;end;
if not(isfield(MConstants,'k_gammaMax')), MConstants.k_gammaMax=3e-2;end;
if not(isfield(MConstants,'k_deltaMax')), MConstants.k_deltaMax=6e3;end;
if not(isfield(MConstants,'k_epsilonMax')), MConstants.k_epsilonMax=3.5e2;end;
if not(isfield(MConstants,'k_xforwardMax')), MConstants.k_xforwardMax=7e-3;end;
if not(isfield(MConstants,'k_xbackMax')), MConstants.k_xbackMax=1e-1;end;

if not(isfield(MConstants,'E_alpha')), MConstants.E_alpha=0.7;end;
if not(isfield(MConstants,'E_gamma')), MConstants.E_gamma=0.7;end;

if not(isfield(MConstants,'CA_Emeasure')), MConstants.CA_Emeasure=0.4;end;
if not(isfield(MConstants,'CA_PretreatTime')), MConstants.CA_PretreatTime=500;end;
if not(isfield(MConstants,'CA_HoldTime')), MConstants.CA_HoldTime=500;end;
if not(isfield(MConstants,'CA_ReactTime')), MConstants.CA_ReactTime=500;end;
if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

if not(isfield(MConstants,'CV_Estart')), MConstants.CV_Estart=0.9;end;
if not(isfield(MConstants,'CV_Eend')), MConstants.CV_Eend=0.4;end;
if not(isfield(MConstants,'CV_ScanRate')), MConstants.CV_ScanRate=0.01;end;
if not(isfield(MConstants,'CV_NScans')), MConstants.CV_NScans=3;end;

if not(isfield(MConstants,'pKa_alpha')), MConstants.pKa_alpha=4.6;end;
if not(isfield(MConstants,'pKa_beta')), MConstants.pKa_beta=4.6;end;
if not(isfield(MConstants,'pKa_gamma')), MConstants.pKa_gamma=20;end;
if not(isfield(MConstants,'pKa_delta')), MConstants.pKa_delta=20;end;
if not(isfield(MConstants,'pKa_epsilon')), MConstants.pKa_epsilon=20;end;
if not(isfield(MConstants,'pKa_x')), MConstants.pKa_x=5.1;end;

if not(isfield(MConstants,'InactiveFxn')), MConstants.InactiveFxn=[0.55,4,-0.02,0.02,5.5];end;

if not(isfield(MConstants,'TimeStep')), MConstants.TimeStep=1e-3;end;
if not(isfield(MConstants,'SteadyState')), MConstants.SteadyState=0;end;
if not(isfield(MConstants,'ShowFigure')), MConstants.ShowFigure=0;end;
if not(isfield(MConstants,'SaveFigure')), MConstants.SaveFigure=0;end;

start=clock;
fprintf('\nStarted at %s\n',datestr(start));

% Where the hits will be saved, put start and header
fileID=fopen(['Hits_',datestr(start,'yyyymmddHHMMSS'),'_P_deadend_tight_xback_sig_pKa',sprintf('-a%4.2f-b%4.2f-g%4.2f-d%4.2f-e%4.2f-x%4.2f',MConstants.pKa_alpha,MConstants.pKa_beta,MConstants.pKa_gamma,MConstants.pKa_delta,MConstants.pKa_epsilon,MConstants.pKa_x),'.csv'],'w'); % **SCHEME-DEPENDENT CODE (1/15)**
fprintf(fileID,'Started at,%s\n',datestr(start));
fprintf(fileID,'folder,pKa_alpha,pKa_beta,pKa_gamma,pKa_delta,pKa_epsilon,pKa_x,k_alphaMax,k_betaforward,k_betaback,k_gammaMax,k_deltaMax,k_epsilonMax,k_xforwardMax,k_xbackMax,');
fprintf(fileID,'Test 1 E_cat(pH 4),Test 2 E_cat(pH 6),Test 3 CV activation,Test 4 CA activation pH 4 100 s hold,Test 5 CA activation slope pH 6 500 s hold,Test 6 pH 6 i(1520)/i(lim),Test 7 CA activation slope pH 4 500 s hold,Test 8 CA activation pH 4 500 s hold,pH 4 i(1520)/i(lim)\n');

% Starting values
factor=10^Order;
growth=10^(1/floor(LogStepSize));
hit=1; % Goes to zero if any test is failed.
Step=1;
TotalSteps=floor((1+Order*2*LogStepSize)^4);
fprintf('%u steps total\n',TotalSteps);
fail=zeros(11,1); % Keep track of where tests are failed
pass=zeros(9,1); % Keep track of the range of test values for hits
test=zeros(4,1); % Set up space for where test values are stored

% Fudge in values to get loops to work properly; otherwise it skips the max
alphaMax=MConstants.k_alphaMax*factor*1.001;
betaMax=MConstants.k_betaforward*factor*1.001;
gammaMax=MConstants.k_gammaMax*factor*1.001;
xbackMax=MConstants.k_xbackMax*factor*1.001;
alphaMin=MConstants.k_alphaMax/factor;
betaMin=MConstants.k_betaforward/factor;
gammaMin=MConstants.k_gammaMax/factor;
xbackMin=MConstants.k_xbackMax/factor;
MConstants.k_alphaMax=alphaMin;
MConstants.k_betaforward=betaMin;
MConstants.k_gammaMax=gammaMin;
MConstants.k_xbackMax=xbackMin;
MConstants.f = MConstants.alpha*96485.34/MConstants.T/8.314472; % in V^-1

% Code speed up
oldShowFigure=MConstants.ShowFigure;
MConstants.ShowFigure=0;
MConstants.CA_PretreatTime=0;
MConstants.SteadyState=1;
MConstants.CA_ReactTime=50;
MConstants.CV_NScans=2;

% Loops
while MConstants.k_alphaMax<=alphaMax
    while MConstants.k_betaforward<=betaMax
        while MConstants.k_gammaMax<=gammaMax
            while MConstants.k_xbackMax<=xbackMax
                % Work out k_betaback and k_xforward to give correct
                % inactive fractions at defined low and high pH values
                [MConstants.k_betaback,MConstants.k_xforwardMax,FlagCode]=find_betaback_xforward(MConstants);

                % Tests: a TRUE value in test is a FAIL, hit is set to
                % zero to prevent further tests, and fail vector is
                % incremented one for the test number

                % Test 9: xforward/betaforward values don't converge to a
                % useful pair
                % Test 10: xforward or betaforward is negative
                % Test 11: xforward or betaforward is imaginary
                if hit && FlagCode
                    hit=0;
                    fail(8+FlagCode)=fail(8+FlagCode)+1;
                end
                    
                % Test 1: E_cat at pH 4 must be between 0.70 and 0.76 V
                % Test 2: E_cat at pH 6 must be between 0.64 and 0.69 V
                % Test 3: activation test at pH 4. Low-potential vertex on
                % first scan of CV must be between 80 and 90% of that
                % from the second scan (wide = 0.73-0.89, expt = ~85%)
                if hit
                   MConstants.pH=4;
                   [Potential4,Current,Deriv4,~,~,~,~,~,~]=BMCO_CV_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (2/15)**
                   test(1)=Current(floor((length(Current)-1)*(1-.5)/MConstants.CV_NScans)); % Vertex current, scan 1
                   test(2)=Current(floor((length(Current)-1)*(2-.5)/MConstants.CV_NScans)); % Vertex current, scan 2
                   MConstants.pH=6;
                   MConstants.CV_NScans=1;
                   [Potential6,~,Deriv6,~,~,~,~,~,~]=BMCO_CV_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (3/15)**
                   MConstants.CV_NScans=2;
                   pass(1)=Potential4(find(Deriv4==max(Deriv4),1)); % Max in derivative from first scan at pH 4
                   pass(2)=Potential6(find(Deriv6==max(Deriv6),1)); % Max in derivative from first scan at pH 6
                   pass(3)=test(1)/test(2);
                   if pass(1)<0.70 || pass(1)>0.76
                       hit=0;
                       fail(1)=fail(1)+1;
                   end
                   if hit && (pass(2)<0.64 || pass(2)>0.69)
                       hit=0;
                       fail(2)=fail(2)+1;
                   end
                   if hit && (pass(3)<0.80 || pass(3)>0.90)
                       hit=0;
                       fail(3)=fail(3)+1;
                   end
                end

                % Test 4: CA inactivation test at pH 4. Resting Oxidized
                % fraction at 100 s, as measured experimentally (5 s 
                % post step), must be between 0.02 and 0.12 (wide = -0.01
                % to 0.23)
                if hit
                   MConstants.pH=4;
                   MConstants.CA_HoldTime=100;
                   [~,Current,~,~,~,~,~,~,~]=BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (4/15)**
                   [~,~,~,~,~,test(1)]=BMCO_SS_P_deadend(MConstants,MConstants.CA_Emeasure); % Get the steady-state current **SCHEME-DEPENDENT CODE (5/15)**
                   pass(4)=1-Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+5)/MConstants.CA_ExpTimeStep))/test(1);
                   if pass(4)<0.02 || pass(4)>0.12
                       hit=0;
                       fail(4)=fail(4)+1;
                   end
                end

                % Test 5: At pH 6, CA inactivation, after 500 s hold
                % time, the ratio between (a) the difference between 
                % steady-state current and the current 50 s after step
                % to (b) the difference between steady-state current
                % and the current 5 s after step must be between 0.12 and
                % and 0.60 (wide=-0.16 to 2.32, exp: two half-lives=~60 s, 
                % this assumes x2 and /2)
                % Test 6: From the same simulation, 1 minus the current
                % at 20 s post hold over steady-state current must be 
                % between -0.28 and -0.12 (wide=-0.30 to -0.044, exp=-0.2
                % sd ~0.04)
                if hit
                   MConstants.pH=6;
                   MConstants.CA_HoldTime=500;
                   [~,~,~,~,~,test(1)]=BMCO_SS_P_deadend(MConstants,MConstants.CA_Emeasure); % Get the steady-state current **SCHEME-DEPENDENT CODE (6/15)**
                   [~,Current,~,~,~,~,~,~,~]=BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (7/15)**
                   test(2)=Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+5)/MConstants.CA_ExpTimeStep)); % Current 5 s post step
                   test(3)=Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+50)/MConstants.CA_ExpTimeStep)); % Current 50 s post step
                   pass(5)=(test(3)-test(1))/(test(2)-test(1));
                   if pass(5)<0.12 || pass(5)>0.60
                       hit=0;
                       fail(5)=fail(5)+1;
                   end

                   % Test 6
                   if hit
                      test(2)=Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+20)/MConstants.CA_ExpTimeStep)); % Current 20 s post step
                      pass(6)=1-test(2)/test(1);
                      if pass(6)<-0.28 || pass(6)>-0.12
                          hit=0;
                          fail(6)=fail(6)+1;
                      end
                   end
                end

                % Test 7: Identical to Test 5, but at pH 4. Range is
                % 0.082 to 0.54 (wide=-0.36 to 0.91, exp: three
                % half-lives=~75s, this range assumes x2 and /2)
                % Test 8: Check that 'inactive fraction' as measured in
                % Fig. 2B is between 0.25 and 0.45 after 500s hold at
                % OC (wide=0.05-0.51, exp=0.30-0.41)
                if hit
                   MConstants.pH=4;
                   MConstants.CA_HoldTime=500;
                   [~,~,~,~,~,test(1)]=BMCO_SS_P_deadend(MConstants,MConstants.CA_Emeasure); % Get the steady-state current **SCHEME-DEPENDENT CODE (8/15)**
                   [~,Current,~,~,~,~,~,~,~]=BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (9/15)**
                   test(2)=Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+5)/MConstants.CA_ExpTimeStep)); % Current 5 s post step
                   test(3)=Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+50)/MConstants.CA_ExpTimeStep)); % Current 50 s post step
                   pass(7)=(test(3)-test(1))/(test(2)-test(1));
                   if pass(7)<0.082 || pass(7)>0.54
                       hit=0;
                       fail(7)=fail(7)+1;
                   end

                   % Test 8
                   if hit
                       pass(8)=1-test(2)/test(1);
                       if pass(8)<0.25 || pass(8)>0.45
                           hit=0;
                           fail(8)=fail(8)+1;
                       else
                           % X-fraction at pH 4
                           pass(9)=1-Current(floor((MConstants.CA_PretreatTime+MConstants.CA_HoldTime+20)/MConstants.CA_ExpTimeStep))/test(1); % Current 20 s post step
                       end
                   end    
               end

                if hit==1 % All tests passed = a hit!
                    % Create a new directory then go there
                    newDir=[sprintf('%07u',Step),'_',datestr(clock,'yyyy-mm-dd-HH-MM-SS')];
                    mkdir(newDir);
                    oldDir=cd(newDir);
                    fprintf('Hit! %s\n',newDir);

                    % Output constants to file
                    fprintf(fileID,'%s,%5.2f,%5.2f,%5.2f,%5.2f,%5.2f,%5.2f,%e,%e,%e,%e,%e,%e,%e,%e',newDir,MConstants.pKa_alpha,MConstants.pKa_beta,MConstants.pKa_gamma,MConstants.pKa_delta,MConstants.pKa_epsilon,MConstants.pKa_x,MConstants.k_alphaMax,MConstants.k_betaforward,MConstants.k_betaback,MConstants.k_gammaMax,MConstants.k_deltaMax,MConstants.k_epsilonMax,MConstants.k_xforwardMax,MConstants.k_xbackMax');
                    fprintf(fileID,',%e',pass);
                    fprintf(fileID,'\n');

                    % Write files, beginning with list of constant
                    save(newDir,'MConstants');

                    if oldShowFigure
                        MConstants.ShowFigure=1;

                        % Show three CVs at pH 4 (Fig. 4)
                        MConstants.CV_NScans=3;
                        MConstants.pH=4;
                        BMCO_CV_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (10/15)**

                        % Generate CAs at pH 4 and 6 (Fig. 3)
                        MConstants.CA_ReactTime=500;
                        MConstants.CA_HoldTime=500;
                        BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (11/15)**
                        MConstants.pH=6;
                        BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (12/15)**

                        % Generate pH profile (Figs 1 and 2)
                        BMCO_pH_profile_P_deadend(MConstants,3.5,6.5,0.5); % **SCHEME-DEPENDENT CODE (13/15)**

                        % Generate inactivation time profile (Fig. 5)
                        BMCO_time_inactive_P_deadend(MConstants,3.5,5.5,0.5,0,3500,500); % **SCHEME-DEPENDENT CODE (14/15)**

                        % Reset to old values
                        MConstants.CA_ReactTime=50;
                        MConstants.CV_NScans=2;
                    end

                    % Back to old directory
                    cd(oldDir);
                end

                % Update to command line on every 1000th step
                if ~mod(Step,1000)
                    fprintf(['At step ',num2str(Step),'/',num2str(TotalSteps),'\n']); 
                end

                % Longer update to command line on every 10000th step
                if ~mod(Step,1e4)
                    fprintf('Started at %s\n',datestr(start));
                    fprintf('Currently at %s\n',datestr(clock));
                    fprintf('Number failing, by test:     ');
                    fprintf('%u\t',fail);
                    fprintf('\nNumber failing, by test (%%): ');
                    fprintf('%3.1f%%\t',fail/Step*100);
                    fprintf('\nPassed %u (%3.1f%%)',Step-sum(fail),(Step-sum(fail))*100/Step);
                    fprintf('\nCurrent test values:\n\tk(alpha)[max]\t%3.1e\n\tk(+beta)\t%3.1e\n\tk(-beta)\t%3.1e\n\tk(gamma)[Max]\t%3.1e\n\tk(+x)   \t%3.1e\n\tk(-x)   \t%3.1e\n',[MConstants.k_alphaMax,MConstants.k_betaforward,MConstants.k_betaback,MConstants.k_gammaMax,MConstants.k_xforwardMax,MConstants.k_xbackMax]);
                end

                % And back to looping along
                hit=1;
                Step=Step+1;
                MConstants.k_xbackMax=MConstants.k_xbackMax*growth;
            end
            MConstants.k_xbackMax=xbackMin;
            MConstants.k_gammaMax=MConstants.k_gammaMax*growth;
        end
        MConstants.k_gammaMax=gammaMin;
        MConstants.k_betaforward=MConstants.k_betaforward*growth;
    end
    MConstants.k_betaforward=betaMin;
    MConstants.k_alphaMax=MConstants.k_alphaMax*growth;
end
MConstants.k_alphaMax=alphaMin; %#ok<STRNU>

% Final report, first to screen, then to file
fprintf('Tested %u\n',TotalSteps);
fprintf('Number failing, by test:     ');
fprintf('%u\t',fail);
fprintf('\nNumber failing, by test (%%): ');
fprintf('%3.1f%%\t',fail/TotalSteps*100);
fprintf('\nPassed %u (%3.1f%%)',TotalSteps-sum(fail),(TotalSteps-sum(fail))*100/TotalSteps);
fprintf('\nStarted at %s\n',datestr(start));
fprintf('Ended at %s\n',datestr(clock));

fprintf(fileID,'\nTested,%u\n',TotalSteps);
fprintf(fileID,',Test 1 i(pH 4)/i(pH 3),Test 2 i(pH 4),Test 3 CV activation,Test 4 CA activation pH 4 100 s hold,Test 5 CA activation slope pH 6 500 s hold,Test 6 pH 6 i(1520)/i(lim),Test 7 CA activation slope pH 4 500 s hold,Test 8 CA activation pH 4 500 s hold,Test 9 xback and betaback do not converge,Test 10 xback or betaback is negative,Test 11 xback or betaback is imaginary\n');
fprintf(fileID,'Number failing by test');
fprintf(fileID,',%u',fail);
fprintf(fileID,'\nNumber failing by test (%%)');
fprintf(fileID,',%3.1f%%',fail/TotalSteps*100);
fprintf(fileID,'\nPassed,%u,%3.1f%%',TotalSteps-sum(fail),(TotalSteps-sum(fail))*100/TotalSteps);
fprintf(fileID,'\nEnded at,%s\n',datestr(clock));

fclose(fileID);
beep;

end



function [betaback,xforward,FlagCode]=find_betaback_xforward(MConstants)

% Iterates to find k_betaback and k_xforward based on fit to Fig. 2B
% inactivation profile. It sets k_betaback based on apparent inactive 
% fraction at low pH (low pH = MConstants.InactiveFxn(2)), then k_xforward 
% based on apparent inactive fraction at high pH (high pH = 
% MConstants.InactiveFxn(5)), iterating until the high potential 
% k_xforward fits in the acceptable range as set by 
% MConstants.InactiveFxn(3:4)
%
% FlagCode:
%   0: normal convergence
%   1: values do not converge
%   2: one or both values are negative
%   3: one or both values are imaginary

xforward = MConstants.k_xforwardMax;
meanhighpHinactive=(MConstants.InactiveFxn(3)+MConstants.InactiveFxn(4))/2;
FlagCode = 0;
test = 999; % so it executes the while loop first time through
step = 0; % Sometimes there is no real solution
maxstep = 200; % Max number of iterations to find xforward/xbetaback pair

% Set values for pH and potential-dependent rate constants and low and high
% pH values
pH = MConstants.InactiveFxn(2);
alphalow = MConstants.k_alphaMax*(1-1/(1+10^(MConstants.pKa_alpha-pH)))/(1.0+exp(MConstants.f*(MConstants.CA_Emeasure-MConstants.E_alpha)));
betaforwardlow = MConstants.k_betaforward*(1-1/(1+10^(MConstants.pKa_beta-pH)));
gammalow = MConstants.k_gammaMax*(1-1/(1+10^(MConstants.pKa_gamma-pH)))/(1.0+exp(MConstants.f*(MConstants.CA_Emeasure-MConstants.E_gamma)));
deltalow = MConstants.k_deltaMax*(1-1/(1+10^(MConstants.pKa_delta-pH)));
epsilonlow = MConstants.k_epsilonMax*(1-1/(1+10^(MConstants.pKa_epsilon-pH)));
xbacklow = MConstants.k_xbackMax*(1-1/(1+10^(MConstants.pKa_x-pH)));

pH = MConstants.InactiveFxn(5);
alphahigh = MConstants.k_alphaMax*(1-1/(1+10^(MConstants.pKa_alpha-pH)))/(1.0+exp(MConstants.f*(MConstants.CA_Emeasure-MConstants.E_alpha)));
betaforwardhigh = MConstants.k_betaforward*(1-1/(1+10^(MConstants.pKa_beta-pH)));
gammahigh = MConstants.k_gammaMax*(1-1/(1+10^(MConstants.pKa_gamma-pH)))/(1.0+exp(MConstants.f*(MConstants.CA_Emeasure-MConstants.E_gamma)));
deltahigh = MConstants.k_deltaMax*(1-1/(1+10^(MConstants.pKa_delta-pH)));
epsilonhigh = MConstants.k_epsilonMax*(1-1/(1+10^(MConstants.pKa_epsilon-pH)));
xbackhigh = MConstants.k_xbackMax*(1-1/(1+10^(MConstants.pKa_x-pH)));

% **BEGIN SCHEME-DEPENDENT SECTION (15/15)**
% Set initial betaback
betaback = -((MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*gammalow*xbacklow+betaforwardlow*((-1+2*MConstants.InactiveFxn(1))*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*(deltalow+epsilonlow)*gammalow)*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*gammalow*xforward+betaforwardlow*deltalow*(gammalow+MConstants.InactiveFxn(1)*(alphalow+gammalow))*xforward-sqrt(-4*betaforwardlow*(MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward)*((-1+MConstants.InactiveFxn(1))*(betaforwardlow*deltalow*epsilonlow+((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+MConstants.InactiveFxn(1)*(alphalow+betaforwardlow)*deltalow*gammalow*xforward)+(((-1+2*MConstants.InactiveFxn(1))*betaforwardlow*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*betaforwardlow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+deltalow*(MConstants.InactiveFxn(1)*alphalow*betaforwardlow+(1+MConstants.InactiveFxn(1))*(alphalow+betaforwardlow)*gammalow)*xforward)^2))/(2*MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+2*(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward));
if betaback<0
   betaback = -((MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*gammalow*xbacklow+betaforwardlow*((-1+2*MConstants.InactiveFxn(1))*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*(deltalow+epsilonlow)*gammalow)*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*gammalow*xforward+betaforwardlow*deltalow*(gammalow+MConstants.InactiveFxn(1)*(alphalow+gammalow))*xforward+sqrt(-4*betaforwardlow*(MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward)*((-1+MConstants.InactiveFxn(1))*(betaforwardlow*deltalow*epsilonlow+((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+MConstants.InactiveFxn(1)*(alphalow+betaforwardlow)*deltalow*gammalow*xforward)+(((-1+2*MConstants.InactiveFxn(1))*betaforwardlow*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*betaforwardlow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+deltalow*(MConstants.InactiveFxn(1)*alphalow*betaforwardlow+(1+MConstants.InactiveFxn(1))*(alphalow+betaforwardlow)*gammalow)*xforward)^2))/(2*MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+2*(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward));
end

while step < maxstep && (test>MConstants.InactiveFxn(4) || test<MConstants.InactiveFxn(3))
    xforward = -(((-betaforwardhigh+meanhighpHinactive*(betaback+betaforwardhigh))*((betaback+betaforwardhigh)*deltahigh*epsilonhigh+(deltahigh*epsilonhigh+betaforwardhigh*(deltahigh+epsilonhigh))*gammahigh+alphahigh*(deltahigh+epsilonhigh)*(betaback+gammahigh))*xbackhigh)/((betaback+meanhighpHinactive*betaback+meanhighpHinactive*betaforwardhigh)*deltahigh*(betaforwardhigh*gammahigh+alphahigh*(betaback+gammahigh))));

    betaback = -((MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*gammalow*xbacklow+betaforwardlow*((-1+2*MConstants.InactiveFxn(1))*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*(deltalow+epsilonlow)*gammalow)*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*gammalow*xforward+betaforwardlow*deltalow*(gammalow+MConstants.InactiveFxn(1)*(alphalow+gammalow))*xforward-sqrt(-4*betaforwardlow*(MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward)*((-1+MConstants.InactiveFxn(1))*(betaforwardlow*deltalow*epsilonlow+((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+MConstants.InactiveFxn(1)*(alphalow+betaforwardlow)*deltalow*gammalow*xforward)+(((-1+2*MConstants.InactiveFxn(1))*betaforwardlow*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*betaforwardlow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+deltalow*(MConstants.InactiveFxn(1)*alphalow*betaforwardlow+(1+MConstants.InactiveFxn(1))*(alphalow+betaforwardlow)*gammalow)*xforward)^2))/(2*MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+2*(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward));
    if betaback<0
       betaback = -((MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*gammalow*xbacklow+betaforwardlow*((-1+2*MConstants.InactiveFxn(1))*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*(deltalow+epsilonlow)*gammalow)*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*gammalow*xforward+betaforwardlow*deltalow*(gammalow+MConstants.InactiveFxn(1)*(alphalow+gammalow))*xforward+sqrt(-4*betaforwardlow*(MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward)*((-1+MConstants.InactiveFxn(1))*(betaforwardlow*deltalow*epsilonlow+((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+MConstants.InactiveFxn(1)*(alphalow+betaforwardlow)*deltalow*gammalow*xforward)+(((-1+2*MConstants.InactiveFxn(1))*betaforwardlow*deltalow*epsilonlow+(-1+MConstants.InactiveFxn(1))*alphalow*betaforwardlow*(deltalow+epsilonlow)+MConstants.InactiveFxn(1)*((alphalow+betaforwardlow)*deltalow+(alphalow+betaforwardlow+deltalow)*epsilonlow)*gammalow)*xbacklow+deltalow*(MConstants.InactiveFxn(1)*alphalow*betaforwardlow+(1+MConstants.InactiveFxn(1))*(alphalow+betaforwardlow)*gammalow)*xforward)^2))/(2*MConstants.InactiveFxn(1)*(deltalow*epsilonlow+alphalow*(deltalow+epsilonlow))*xbacklow+2*(1+MConstants.InactiveFxn(1))*alphalow*deltalow*xforward));
    end

    % Work out 'inactive fraction' at high pH
    test = betaforwardhigh/(betaback+betaforwardhigh)-(deltahigh*(betaforwardhigh*gammahigh+alphahigh*(betaback+gammahigh))*xforward)/(((betaback+betaforwardhigh)*deltahigh*epsilonhigh+(deltahigh*epsilonhigh+betaforwardhigh*(deltahigh+epsilonhigh))*gammahigh)*xbackhigh+betaforwardhigh*deltahigh*gammahigh*xforward+alphahigh*(betaback+gammahigh)*(epsilonhigh*xbackhigh+deltahigh*(xbackhigh+xforward)));

    step = step+1;
end
% **END SCHEME-DEPENDENT SECTION (15/15)**

% I'm not sure if negative or complex values could be flagged up earlier
% (before or during loop, for example) so playing it safe
if step >= maxstep
    FlagCode = 1;
elseif ~isreal(betaback) || ~isreal(xforward)
    FlagCode = 3;
elseif betaback < 0 || xforward < 0
    FlagCode = 2;
end

end
