function [OCTime,IFraction]=BMCO_time_inactive_P_deadend(MConstants, pHstart, pHend, pHstep, timemin, timemax, timestep)

% BMCO_time_inactive
%   Scan over pH and open-circuit time values to simulate or validate 
%   results of chronoamperometry results shown in Fig. 2B (cf. Fig. 7).
%   Generates matching plot in MConstants.ShowFigure=1
%    2011 James A. Cracknell and Christopher F. Blanford
%
%   Input
%       MConstants
%           Structure with simulation variable (see below)
%       pHstart
%           First pH value in range of pH values
%       pHend
%           Maximum pH value (may not be last value depending on pHstep
%           size)
%       pHstep
%           Size of increment in pH values for simulation
%       timemin
%           First length of time for open-circuit hold
%       timemax
%           Maximum length of open-circuit hold time
%       timestep
%           Increment in time held at open-circuit
%
%   Output
%       OCTime
%           Time at open-circuit before applied potential in
%           chronoamperometry measurements (equivalent to x-axis in Fig.
%           2B) (two-dimensional matrix)
%       IFraction
%           Apparent inactive fraction (equivalent to y-axis in Fig. 2B)
%           (two-dimensional matrix)
%
% 	Simulation variables and rate constants (MConstants)
%       .T
%           Absolute temperature in K
%       .pH
%           pH
%       .alpha
%           Electron transfer coefficient
%
%       .EnzConc
%           Surface concentration of enzyme in mol cm^-2
%
%       .k_alphaMax
%           Active Oxidized to Reduced
%       .k_betaforward
%           Active Oxidized to Resting Oxidized
%       .k_betaback
%           Resting Oxidized to Active Oxidized
%       .k_gammaMax
%           Resting Oxidized to Reduced
%       .k_deltaMax
%           Reduced to Peroxy
%       .k_epsilonMax
%           Peroxy to Active Oxidized
%       .k_xforwardMax
%           Peroxy to X state
%       .k_xbackMax
%           X state to Peroxy
%       .E_alpha
%           Catalytic potential Active Oxidized to Reduced in Volts vs SHE
%       .E_gamma
%           Catalytic potential Resting Oxidized to Reduced in Volts vs SHE
%
%       .CV_Estart
%           Start potential for CV in V vs SHE
%       .CV_Eend
%           Vertex potential for CV in V vs SHE
%       .CV_ScanRate
%           Rate of change in potential in V s-1
%       .CV_NScans
%           Number of potential cycles
%
%       .pKa_a
%           pH of average rate for reaction alpha (Active Oxidized to Reduced)
%       .pKa_b
%           pH of average rate for reaction beta (Active Oxidized to Resting Oxidized)
%       .pKa_g
%           pH of average rate for reaction gamma (Inactive to Reduced)
%       .pKa_d
%           pH of average rate for reaction delta (Reduced to Peroxy)
%       .pKa_e
%           pH of average rate for reaction epsilon (Peroxy to Active Oxidized)
%       .pKa_x
%           pH of average rate for reaction -x (Peroxy to X)
%
%       .InactiveFxn
%           Vector of values used to fit k_betaback and k_xforward from
%           steady-state equations and experimental measurements.
%           (1)	apparent inactive fraction at low pH after an "infinite" 
%               time at open circuit
%           (2) low pH value
%           (3) lower acceptable limit for apparent inactive fraction at
%               high pH after an "infinite" time at open circuit
%           (4) corresponding higher acceptable limit
%           (5) high pH value
%
%       .TimeStep
%           Time resolution for finite time difference simulation
%       .SteadyState
%           Start simulation with enzyme fractions based on catalytic
%           steady-state if true (logical 1); otherwise starts with
%           steady-state open-circuit values.
%       .ShowFigure
%           Plot figures in function if set to true (logical 1)
%       .SaveFigure
%           Save figure(s) as png and fig files (CAs only saved as png)


% _______Check for variables in MConstants structure and set defaults if not present________

try
    isstruct(MConstants);
catch ME %#ok<NASGU>
    MConstants=struct;
end

if not(isfield(MConstants,'T')), MConstants.T=298;end;
if not(isfield(MConstants,'pH')), MConstants.pH=4;end;
if not(isfield(MConstants,'alpha')), MConstants.alpha=0.5;end;

if not(isfield(MConstants,'EnzConc')), MConstants.EnzConc=1e-12;end;

if not(isfield(MConstants,'k_alphaMax')), MConstants.k_alphaMax=2.5e3;end;
if not(isfield(MConstants,'k_betaforward')), MConstants.k_betaforward=1e-3;end;
if not(isfield(MConstants,'k_betaback')), MConstants.k_betaback=7.1e-4;end;
if not(isfield(MConstants,'k_gammaMax')), MConstants.k_gammaMax=3e-2;end;
if not(isfield(MConstants,'k_deltaMax')), MConstants.k_deltaMax=6e3;end;
if not(isfield(MConstants,'k_epsilonMax')), MConstants.k_epsilonMax=3.5e2;end;
if not(isfield(MConstants,'k_xforwardMax')), MConstants.k_xforwardMax=7e-3;end;
if not(isfield(MConstants,'k_xbackMax')), MConstants.k_xbackMax=1e-1;end;

if not(isfield(MConstants,'E_alpha')), MConstants.E_alpha=0.7;end;
if not(isfield(MConstants,'E_gamma')), MConstants.E_gamma=0.7;end;

if not(isfield(MConstants,'CA_Emeasure')), MConstants.CA_Emeasure=0.4;end;
if not(isfield(MConstants,'CA_PretreatTime')), MConstants.CA_PretreatTime=500;end;
if not(isfield(MConstants,'CA_HoldTime')), MConstants.CA_HoldTime=500;end;
if not(isfield(MConstants,'CA_ReactTime')), MConstants.CA_ReactTime=500;end;
if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

if not(isfield(MConstants,'CV_Estart')), MConstants.CV_Estart=0.9;end;
if not(isfield(MConstants,'CV_Eend')), MConstants.CV_Eend=0.4;end;
if not(isfield(MConstants,'CV_ScanRate')), MConstants.CV_ScanRate=0.01;end;
if not(isfield(MConstants,'CV_NScans')), MConstants.CV_NScans=3;end;

if not(isfield(MConstants,'pKa_alpha')), MConstants.pKa_alpha=4.6;end;
if not(isfield(MConstants,'pKa_beta')), MConstants.pKa_beta=4.6;end;
if not(isfield(MConstants,'pKa_gamma')), MConstants.pKa_gamma=20;end;
if not(isfield(MConstants,'pKa_delta')), MConstants.pKa_delta=20;end;
if not(isfield(MConstants,'pKa_epsilon')), MConstants.pKa_epsilon=20;end;
if not(isfield(MConstants,'pKa_x')), MConstants.pKa_x=5.1;end;

if not(isfield(MConstants,'InactiveFxn')), MConstants.InactiveFxn=[0.55,4,-0.02,0.02,5.5];end;

if not(isfield(MConstants,'TimeStep')), MConstants.TimeStep=1e-3;end;
if not(isfield(MConstants,'SteadyState')), MConstants.SteadyState=0;end;
if not(isfield(MConstants,'ShowFigure')), MConstants.ShowFigure=0;end;
if not(isfield(MConstants,'SaveFigure')), MConstants.SaveFigure=0;end;


% Zero storage matrices
TotalpHSteps = floor((pHend-pHstart)/pHstep);
if ~mod((pHend-pHstart),pHstep), TotalpHSteps=TotalpHSteps+1;end;

TotaltimeSteps = floor((timemax-timemin)/timestep);
if ~mod((timemax-timemin),timestep), TotaltimeSteps=TotaltimeSteps+1;end;

pH=zeros(TotalpHSteps,1);
PlotLegend=cell(TotalpHSteps,1);
OCTime=zeros(TotaltimeSteps,1);

IFraction=zeros(TotaltimeSteps,TotalpHSteps);

MConstants.pH=pHstart;
MConstants.CA_PretreatTime=0;
MConstants.SteadyState=1;
if not(isfield(MConstants,'SaveFigure'))
    MConstants.SaveFigure=0;
else
    SaveFigure=MConstants.SaveFigure;
    MConstants.SaveFigure=0;
end;
MConstants.f = MConstants.alpha*96485.34/MConstants.T/8.314472; % in V^-1

if not(isfield(MConstants,'CA_ExpTimeStep')), MConstants.CA_ExpTimeStep=0.1;end;

i=1;

while (MConstants.pH<=max(pHstart,pHend) && MConstants.pH>=min(pHstart,pHend))
    pH(i)=MConstants.pH;
    PlotLegend(i)=cellstr(sprintf('pH %g',pH(i)));
    
    j=1;
    holdtime=timemin;
    
    while holdtime<=timemax
        OCTime(j,i)=holdtime;
        MConstants.CA_HoldTime=holdtime;
        % Chronoamperometry data points
        [~,Current,~,~,~,~,~,~,Const]=BMCO_CA_P_deadend(MConstants); % **SCHEME-DEPENDENT CODE (1/3)**
        [~,~,~,~,~,SSCurrent]=BMCO_SS_P_deadend(MConstants,MConstants.CA_Emeasure); % **SCHEME-DEPENDENT CODE (2/3)**
        IFraction(j,i)=1-(Current(floor(Const.CA_PretreatTime+Const.CA_HoldTime+5)/MConstants.CA_ExpTimeStep)/SSCurrent);

        holdtime=MConstants.CA_HoldTime+timestep;
        j=j+1;

    end

    % Increment
    MConstants.pH=MConstants.pH+pHstep;
    i=i+1;
        
end


% Generate CA figure

MConstants.SaveFigure=SaveFigure;
figure(5);
clf;
plot(OCTime,IFraction,'-o')
xlabel('Time at open circuit / s');
ylabel('i_{(step+5s)}/i_{lim}');
infotext={['pK_{a,\alpha} = ', num2str(MConstants.pKa_alpha),'  pK_{a,\beta} = ',num2str(MConstants.pKa_beta),' pK_{a,\gamma} = ',num2str(MConstants.pKa_gamma),' pK_{a,\delta} = ',num2str(MConstants.pKa_delta),'  pK_{a,\epsilon} = ',num2str(MConstants.pKa_epsilon),'  pK_{a,x} = ',num2str(MConstants.pKa_x)];['k_{\alpha} = ', num2str(MConstants.k_alphaMax,'%g'),'  k_{\beta} = ',num2str(MConstants.k_betaforward,'%g'),'  k_{-\beta}= ',num2str(MConstants.k_betaback,'%g'),'  k_{\gamma} = ',num2str(MConstants.k_gammaMax,'%g')];['k_{\delta} = ',num2str(MConstants.k_deltaMax,'%g'),'  k_{\epsilon} = ',num2str(MConstants.k_epsilonMax,'%g'),'  k_{x} = ',num2str(MConstants.k_xforwardMax,'%g'),' k_{-x} = ',num2str(MConstants.k_xbackMax,'%g')];['E_{\alpha} = ', num2str(MConstants.E_alpha),'  E_{\gamma} = ',num2str(MConstants.E_gamma)]};
title('Fraction inactivated after x seconds at OC (deadend from P scheme)','FontSize',10) % **SCHEME-DEPENDENT CODE (3/3)**
legend(PlotLegend,'Location','NorthWest');
text(0,-0.2,infotext,'FontSize',8)
if MConstants.SaveFigure
    filename=['Fig5_',datestr(clock,'yyyy-mm-dd-HH-MM-SS')];
    saveas(gca,filename,'fig')
    saveas(gca,filename,'png')
end

end